CREATE TYPE TypeVehicule AS ENUM ('Hybride', 'Berline', 'Camionnette');
CREATE DOMAIN TypeImmatriculation AS CHAR(7) NOT NULL;
CREATE DOMAIN TypeNumeroPermis AS CHAR(13) NOT NULL;
CREATE DOMAIN TypeCodePostal AS CHAR(6) NOT NULL;
CREATE DOMAIN TypeTexteCourt AS VARCHAR(50) NOT NULL;
CREATE DOMAIN TypeTexteCourtOptionnel AS VARCHAR(50);
CREATE DOMAIN TypeTexteLong AS VARCHAR(200) NOT NULL;
CREATE DOMAIN TypeTexteLongOptionnel AS VARCHAR(200);
CREATE DOMAIN Url as TEXT;

CREATE TABLE emplacement (
    nom TypeTexteCourt NOT NULL,
    nb_stationnements INT NOT NULL,
    adresse_postale_ligne_principale TypeTexteLong,
    adresse_postale_ligne_secondaire TypeTexteLongOptionnel,
    adresse_postale_ville TypeTexteCourt,
    adresse_postale_cp TypeCodePostal,
    carte_url Url,
    PRIMARY KEY (nom)
);

CREATE TABLE vehicule (
    immatriculation TypeImmatriculation,
    marque TypeTexteCourt,
    modele TypeTexteCourt,
    odometre INT NOT NULL,
    date_mise_en_service DATE NOT NULL,
    consommation FLOAT NOT NULL,
    "type" TypeVehicule NOT NULL,
    kw_recharge FLOAT,
    emplacement_nom TypeTexteCourt,
    PRIMARY KEY (immatriculation),
    FOREIGN KEY (emplacement_nom) REFERENCES emplacement (nom)
);

CREATE TABLE vehicule_specificites (
    nom TypeTexteCourt NOT NULL,
    immatriculation TypeImmatriculation,
    PRIMARY KEY (nom, immatriculation),
    FOREIGN KEY (immatriculation) REFERENCES vehicule (immatriculation)
);

CREATE TABLE assurance (
    numero INT NOT NULL,
    assureur TypeTexteCourt,
    date_debut DATE NOT NULL,
    date_echeance DATE NOT NULL,
    immatriculation TypeImmatriculation,
    PRIMARY KEY (numero),
    FOREIGN KEY (immatriculation) REFERENCES vehicule (immatriculation)
);

CREATE TABLE membre (
    numero_membre INT NOT NULL,
    "password" TypeTexteLong,
    montant_parts INT NOT NULL,
    nom TypeTexteLong,
    courriel TypeTexteLong,
    adresse_postale_ligne_principale TypeTexteLong,
    adresse_postale_ligne_secondaire TypeTexteLongOptionnel,
    adresse_postale_ville TypeTexteCourt,
    adresse_postale_cp TypeCodePostal,
    compte_bancaire_numero_institution INT NOT NULL,
    compte_bancaire_numero_transit INT NOT NULL,
    compte_bancaire_numero_compte INT NOT NULL,
    est_membre_physique BOOLEAN NOT NULL,
    cotisation_reduite BOOLEAN,
    emplacement_favoris_nom TypeTexteCourtOptionnel,
    PRIMARY KEY (numero_membre),
    FOREIGN KEY (emplacement_favoris_nom) REFERENCES emplacement (nom)
);

CREATE TABLE conducteur (
    numero_permis TypeNumeroPermis,
    nom_complet TypeTexteLong,
    numero_membre INT NOT NULL,
    PRIMARY KEY (numero_permis, numero_membre),
    FOREIGN KEY (numero_membre) REFERENCES membre (numero_membre)
);

CREATE TABLE reservation (
    temps_debut TIMESTAMP NOT NULL,
    temps_fin TIMESTAMP NOT NULL,
    numero_permis_conducteur TypeNumeroPermis,
    numero_membre INT NOT NULL,
    immatriculation TypeImmatriculation,
    PRIMARY KEY (temps_debut, numero_permis_conducteur, numero_membre, immatriculation),
    FOREIGN KEY (numero_permis_conducteur, numero_membre) REFERENCES conducteur (numero_permis, numero_membre),
    FOREIGN KEY (immatriculation) REFERENCES vehicule (immatriculation)
);

CREATE TABLE facture (
    date_creation DATE NOT NULL,
    date_echeance DATE NOT NULL,
    total FLOAT NOT NULL,
    est_payee BOOLEAN NOT NULL DEFAULT FALSE,
    numero_membre INT NOT NULL,
    PRIMARY KEY (date_creation, numero_membre),
    FOREIGN KEY (numero_membre) REFERENCES membre (numero_membre)
);

CREATE TABLE utilisation (
    temps_debut TIMESTAMP NOT NULL,
    temps_fin TIMESTAMP NOT NULL,
    odometre_debut INT NOT NULL,
    odometre_fin INT NOT NULL,
    numero_membre INT NOT NULL,
    immatriculation TypeImmatriculation,
    date_creation_facture DATE,
    PRIMARY KEY (temps_debut, numero_membre, immatriculation),
    FOREIGN KEY (numero_membre) REFERENCES membre (numero_membre),
    FOREIGN KEY (immatriculation) REFERENCES vehicule (immatriculation),
    FOREIGN KEY (date_creation_facture, numero_membre) REFERENCES facture (date_creation, numero_membre)
);

CREATE TABLE caracteristiques_vehicule (
    prix_horaire FLOAT NOT NULL,
    prix_kilometrique FLOAT NOT NULL,
    odometre_max INT NOT NULL,
    "type" TypeVehicule NOT NULL,
    PRIMARY KEY ("type")
);
