import { Membre as MembreInterface } from '@common/interfaces/membre';
import { ApiHideProperty } from '@nestjs/swagger';
import { Exclude } from 'class-transformer';
import { MembreRow } from 'rows/membre.row';
import { AdressePostale } from './adresse-postale.entity';
import { CompteBancaire } from './compte-bancaire.entity';
import { Emplacement } from './emplacement.entity';

export class Membre implements MembreInterface {
  numeroMembre: number;

  @Exclude()
  @ApiHideProperty()
  password: string;

  montantParts: number;
  nom: string;
  courriel: string;
  adressePostale: AdressePostale;

  compteBancaire: CompteBancaire;
  emplacementFavoris: Emplacement | null;

  constructor(membreRow: MembreRow, emplacementFavoris: Emplacement | null) {
    this.numeroMembre = membreRow.numero_membre;
    this.password = membreRow.password;
    this.montantParts = membreRow.montant_parts;
    this.nom = membreRow.nom;
    this.courriel = membreRow.courriel;
    this.adressePostale = new AdressePostale(
      membreRow.adresse_postale_ligne_principale,
      membreRow.adresse_postale_ville,
      membreRow.adresse_postale_cp,
      membreRow.adresse_postale_ligne_secondaire,
    );
    this.compteBancaire = new CompteBancaire(
      membreRow.compte_bancaire_numero_institution,
      membreRow.compte_bancaire_numero_transit,
      membreRow.compte_bancaire_numero_compte,
    );
    this.emplacementFavoris = emplacementFavoris;
  }
}

export class MembrePhysique extends Membre {
  cotisationReduite: boolean;

  constructor(membreRow: MembreRow, emplacementFavoris: Emplacement | null) {
    super(membreRow, emplacementFavoris);
    this.cotisationReduite = membreRow.cotisation_reduite as boolean;
  }
}
