import {
  Controller,
  Get,
  HttpException,
  HttpStatus,
  Param,
  Query,
} from '@nestjs/common';
import { ApiQuery, ApiTags } from '@nestjs/swagger';
import { DriverService } from '@src/services/driver.service';
import { MemberService } from '@src/services/member.service';
import { Conducteur } from 'model/conducteur.entity';
import { Membre } from 'model/membre.entity';

@Controller('members')
@ApiTags('members')
export class MemberController {
  constructor(
    private memberService: MemberService,
    private driverService: DriverService,
  ) {}

  @Get('/')
  @ApiQuery({ name: 'name', required: false })
  async find(@Query('name') name?: string): Promise<Membre[]> {
    if (name) {
      return await this.memberService.search(name);
    }
    return await this.memberService.findAll();
  }

  @Get('/:numeroMembre')
  async get(@Param('numeroMembre') numeroMembre: number): Promise<Membre> {
    const membre = await this.memberService.findByNumeroMembre(numeroMembre);
    if (!membre) {
      throw new HttpException('Membre non trouvé', HttpStatus.NOT_FOUND);
    }
    return membre;
  }

  @Get('/:numeroMembre/drivers')
  async getDrivers(
    @Param('numeroMembre') numeroMembre: number,
  ): Promise<Conducteur[]> {
    const membre = await this.get(numeroMembre);
    return this.driverService.findByMembre(membre);
  }
}
