import {
  Body,
  Controller,
  Get,
  HttpException,
  HttpStatus,
  Post,
} from '@nestjs/common';
import { ApiTags } from '@nestjs/swagger';
import { DriverService } from '@src/services/driver.service';
import { EmplacementService } from '@src/services/emplacement.service';
import { MemberService } from '@src/services/member.service';
import { ReservationService } from '@src/services/reservation.service';
import { CreateReservationDto } from 'dto/create-reservation.dto';
import { Reservation } from 'model/reservation.entity';

@Controller('reservations')
@ApiTags('reservations')
export class ReservationController {
  constructor(
    private reservationService: ReservationService,
    private emplacementService: EmplacementService,
    private memberService: MemberService,
    private driverService: DriverService,
  ) {}

  @Get('/')
  async findAll(): Promise<Reservation[]> {
    return await this.reservationService.findAll();
  }

  @Post('/')
  async create(@Body() createReservationDto: CreateReservationDto) {
    const emplacement = await this.emplacementService.findByName(
      createReservationDto.emplacementNom,
    );
    if (!emplacement) {
      throw new HttpException('Emplacement non trouvé', HttpStatus.NOT_FOUND);
    }
    const membre = await this.memberService.findByNumeroMembre(
      createReservationDto.numeroMembre,
    );
    if (!membre) {
      throw new HttpException('Membre non trouvé', HttpStatus.NOT_FOUND);
    }
    const conducteur = await this.driverService.findOneByNumeroPermisAndMembre(
      createReservationDto.numeroPermisConducteur,
      membre,
    );
    if (!conducteur) {
      throw new HttpException('Driver not found', HttpStatus.NOT_FOUND);
    }

    return await this.reservationService.tryCreate(
      emplacement,
      createReservationDto.typeVehicule,
      conducteur,
      createReservationDto.tempsDebut,
      createReservationDto.tempsFin,
    );
  }
}
