import { Injectable } from '@nestjs/common';
import { Pool, QueryResult, QueryResultRow } from 'pg';

@Injectable()
export class DatabaseService {
  private readonly connexionConfig = {
    user: process.env.DATABASE_USER || 'postgres',
    database: process.env.DATABASE_NAME || 'postgres',
    password: process.env.DATABASE_PASSWORD || 'postgres',
    host: process.env.DATABASE_HOST || 'localhost',
    port: Number(process.env.DATABASE_PORT) || 5432,
    keepAlive: true,
  };

  private pool: Pool = new Pool(this.connexionConfig);

  public async query<T extends QueryResultRow>(
    query: string,
    params?: unknown[],
  ): Promise<QueryResult<T>> {
    const client = await this.pool.connect();
    const response = await client.query<T>(query, params);
    client.release();
    return response;
  }
}
