import { Injectable } from '@nestjs/common';
import { Conducteur } from 'model/conducteur.entity';
import { Membre } from 'model/membre.entity';
import { ConducteurRow } from 'rows/conducteur.row';
import { DatabaseService } from './database.service';
import { MemberService } from './member.service';

@Injectable()
export class DriverService {
  constructor(
    private databaseService: DatabaseService,
    private memberService: MemberService,
  ) {}

  async findByMembre(membre: Membre): Promise<Conducteur[]> {
    const query = `SELECT * from conducteur where numero_membre = $1`;
    const result = await this.databaseService.query<ConducteurRow>(query, [
      membre.numeroMembre,
    ]);
    return await Promise.all(
      result.rows.map((row) => new Conducteur(row, membre)),
    );
  }

  async findOneByNumeroPermisAndMembre(
    numeroPermis: string,
    membre: Membre,
  ): Promise<Conducteur | null> {
    const query = `SELECT * from conducteur WHERE numero_permis = $1 AND numero_membre = $2`;
    const result = await this.databaseService.query<ConducteurRow>(query, [
      numeroPermis,
      membre.numeroMembre,
    ]);
    if (!result.rowCount) return null;
    return new Conducteur(result.rows[0], membre);
  }
}
