import { Injectable } from '@nestjs/common';
import { Emplacement } from 'model/emplacement.entity';
import { Membre, MembrePhysique } from 'model/membre.entity';
import { MembreRow } from 'rows/membre.row';
import { DatabaseService } from './database.service';
import { EmplacementService } from './emplacement.service';

@Injectable()
export class MemberService {
  constructor(
    private databaseService: DatabaseService,
    private emplacementService: EmplacementService,
  ) {}

  async findAll(): Promise<(Membre | MembrePhysique)[]> {
    const query = `SELECT * FROM membre`;
    const result = await this.databaseService.query<MembreRow>(query);
    return await Promise.all(
      result.rows.map(async (row) => {
        return await this.createMembreFromRow(row);
      }),
    );
  }

  async search(name: string): Promise<(Membre | MembrePhysique)[]> {
    const query = `SELECT * FROM membre WHERE LOWER(nom) LIKE LOWER($1)`;
    const result = await this.databaseService.query<MembreRow>(query, [
      `%${name}%`,
    ]);
    return await Promise.all(
      result.rows.map(async (row) => {
        return await this.createMembreFromRow(row);
      }),
    );
  }

  async findByNumeroMembre(
    numeroMembre: number,
  ): Promise<Membre | MembrePhysique | null> {
    const query = `SELECT * FROM membre WHERE numero_membre = $1`;
    const result = await this.databaseService.query<MembreRow>(query, [
      numeroMembre,
    ]);
    if (!result.rowCount) return null;
    const row = result.rows[0];
    return await this.createMembreFromRow(row);
  }

  private async createMembreFromRow(
    membreRow: MembreRow,
  ): Promise<Membre | MembrePhysique> {
    let emplacementFavoris: Emplacement | null = null;
    if (membreRow.emplacement_favoris_nom) {
      emplacementFavoris = await this.emplacementService.findByName(
        membreRow.emplacement_favoris_nom,
      );
    }
    return membreRow.est_membre_physique
      ? new MembrePhysique(membreRow, emplacementFavoris)
      : new Membre(membreRow, emplacementFavoris);
  }
}
