import { TypeVehicule } from '@common/enum/type-vehicule.enum';
import { Injectable } from '@nestjs/common';
import { Emplacement } from 'model/emplacement.entity';
import { Vehicule, VehiculeHybride } from 'model/vehicule.entity';
import { VehiculeRow } from 'rows/vehicule.row';
import { DatabaseService } from './database.service';
import { EmplacementService } from './emplacement.service';

@Injectable()
export class VehicleService {
  constructor(
    private databaseService: DatabaseService,
    private emplacementService: EmplacementService,
  ) {}

  async findOneByImmatriculation(
    immatriculation: string,
  ): Promise<Vehicule | VehiculeHybride | null> {
    const query = `SELECT * from vehicule WHERE immatriculation = $1`;
    const result = await this.databaseService.query<VehiculeRow>(query, [
      immatriculation,
    ]);
    if (!result.rowCount) return null;
    const row = result.rows[0];
    const emplacement = row.emplacement_nom
      ? await this.emplacementService.findByName(row.emplacement_nom)
      : null;

    return this.createVehicleFromRow(row, emplacement);
  }

  async findAvailableByEmplacementAndType(
    emplacement: Emplacement,
    type: TypeVehicule,
    tempsDebut: Date,
    tempsFin: Date,
  ): Promise<(Vehicule | VehiculeHybride)[]> {
    const query =
      'SELECT *, vehicule.immatriculation from vehicule ' +
      'LEFT OUTER JOIN reservation ON vehicule.immatriculation = reservation.immatriculation ' +
      'AND (' +
      '(reservation.temps_debut BETWEEN $3 AND $4) OR ' +
      '(reservation.temps_fin BETWEEN $3 AND $4) OR ' +
      '(reservation.temps_debut <= $3 AND reservation.temps_fin >= $4)) ' +
      'WHERE ' +
      'vehicule.emplacement_nom = $1 AND ' +
      'vehicule.type = $2 AND ' +
      'reservation.temps_debut IS NULL';
    const result = await this.databaseService.query<VehiculeRow>(query, [
      emplacement.nom,
      type,
      tempsDebut,
      tempsFin,
    ]);
    return await Promise.all(
      result.rows.map(async (row) => {
        return this.createVehicleFromRow(row, emplacement);
      }),
    );
  }

  private async createVehicleFromRow(
    row: VehiculeRow,
    emplacement: Emplacement | null,
  ): Promise<Vehicule | VehiculeHybride> {
    if (row.type === TypeVehicule.Hybride) {
      return new VehiculeHybride(row, emplacement);
    }
    return new Vehicule(row, emplacement);
  }
}
